
/************************************************************************
* Copyright (c) 1995 Valentin Shopov, Blagoevgrad, Bulgaria		*
*									*
* Permission to use, copy, modify, and distribute this material for any	*
* purpose and without fee is hereby granted, provided that the above	*
* copyright notice and this permission notice appear in all copies, and	*
* that the name of the author not be used in advertising or publicity 	*
* pertaining to this material without the specific, prior written 	*
* permission of the author.						*
*									*
* THE AUTHOR MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY	*  
* OF THIS MATERIAL FOR ANY PURPOSE. IT IS PROVIDED "AS IS", WITHOUT ANY	* 
* EXPRESS OR IMPLIED WARRANTIES.					*
*************************************************************************/

/**************************ciilib1.4.0***********************************
 *                                                                      *
 *        Character to Inline Image Conversion Library (CIILIB)         *
 *                                                                      *
 *         x16 and x14 BDF Font to CIILIB XBM source convertor.		*
 *              	by Valentin Shopov		                *
 *			valentin@rs.aubg.bg				*
 ************************************************************************/





#include <stdio.h>
#include <string.h>
#include <memory.h>
#include "cii.h"

char curcol[2048], currow[2048];
int KANIND[0xffff];
static char *errlog = "bdf2xbmerr.log";
static char *info = "bdf2xbm.inf";

struct CharCodesXBM SupChC[] =
{
  {
    "jp",
    "JISX0208",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "cyrillic",
    "ISO8859-5",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "greek",
    "ISO8859-7",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "hebrew",
    "ISO8859-8",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "chinese",
    "GB2312",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "korean",
    "KSC5601",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "koi8",
    "GOST19768",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "lat1",
    "ISO8859-1",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "lat2",
    "ISO8859-2",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "lat3",
    "ISO8859-3",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "lat4",
    "ISO8859-4",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }
  ,
  {
    "lat5",
    "ISO8859-9",
    NULL, NULL,
    0, 0, 0, 0, 0, 0
  }


};


char *
getcol (row, numcol)
     char *row;
     int numcol;
{
  int i, j, k;
  char *tmpcol;
  if (numcol <= 0)
    return row;
  tmpcol = curcol;
  for (i = 0, j = 0, k = 0; row[i] != '\n'; i++)
    {
      if (row[i] == ' ' || row[i] == '\t')
	{
	  if (j)
	    {
	      tmpcol[j] = '\0';
	      j = 0;
	      if (++k == numcol)
		return tmpcol;
	    }
	  continue;
	}
      tmpcol[j++] = row[i];
    }
  if (k == numcol - 1)
    {
      tmpcol[j] = '\0';
    }
  else
    tmpcol[0] = '\0';
  return tmpcol;
}

char *
strcsstr (str1, str2)
     char *str1, *str2;
{
  int i, j;
  for (i = 0; i < strlen (str1) - strlen (str2) + 1; i++)
    {
      for (j = 0; j < strlen (str2); j++)
	{
	  if (str2[j] == str1[i + j] & 0xdf)
	    {
	      if (j == strlen (str2) - 1)
		return str1 + i;
	    }
	  else
	    break;
	}
    }
  return 0;
}


mirror (src)
     char *src;
{
  int bmrow, i, lbit = 0x0001, hbit = 0x0080, hlbit = 0x0100, hhbit = 0x8000,
    dst;
  sscanf (src, "%x", &bmrow);
  for (i = 0, dst = 0; i < 8; i++)
    {
      dst |= (lbit & bmrow) ? hbit : 0;
      dst |= (hlbit & bmrow) ? hhbit : 0;
      lbit <<= 1;
      hbit >>= 1;
      hlbit <<= 1;
      hhbit >>= 1;
    }
  return dst;
}


main (ac, av)
     char *av[];
{
  unsigned int XBMPOS, NOFCH, NOFB, CHAR, ROW, KINROW, BMB, cnvtb, x;
  int width, height, fc;
  int ccnum, initc;
  unsigned int first, last;
  char *row;
  FILE *dstf, *srcf, *infof, *errf;
  int tabsize;
  errf = fopen (errlog, "w+");
  if (errf == NULL)
    {
      fprintf (stderr, "Error opening \"%s\"!", errlog);
      exit (1);
    }
  printf ("Writing errors to \"%s\"...\n", errlog);

  if (ac < 2)
    {
      fprintf (errf, "Usage: %s  <x16 or x14 bdf-file-name>\n", av[0]);
      exit (1);
    }

  row = currow;
  printf ("Creating \"_xbmraw.c\"...\n");

  dstf = fopen ("_xbmraw.c", "w+");
  if (dstf == NULL)
    {
      fprintf (errf, "Error opening \"_xbmraw.c\"!");
      exit (1);
    }

  infof = fopen (info, "w+");
  if (infof == NULL)
    {
      fprintf (errf, "Error opening \"%s\"!", info);
      exit (1);
    }

  printf ("Writing info to \"%s\"...\n", info);



  fprintf (infof, "\nList of processed fonts:\n\n");
  fprintf (infof, "%25s%15s%18s%7s%7s%7s\n", "BDF File", "Name", "Register Name", "Width", "Height", "Chars");

  fprintf (dstf, "#include \"cii.h\"\n");






  tabsize = 0;
  for (fc = 1; fc < ac; fc++)
    {
      if (av[fc][0] == '+')
	{
	  tabsize += atoi (&av[fc][1]);
	  continue;
	}
      tabsize++;

      initc = 0;
      XBMPOS = 0;
      BMB = 0;

      for (x = 0; x < 0xffff; x++)
	KANIND[x] = -1;

      srcf = fopen (av[fc], "r");
      if (srcf == NULL)
	{
	  fprintf (errf, "Error opening \"%s\"!", av[fc]);
	  exit (1);
	}
      printf ("Processing %s ....\n", av[fc]);
      fprintf (errf, "Processing %s ....\n", av[fc]);

      for (; fgets (row, 2048, srcf) != NULL;)
	{
	  if (!XBMPOS)
	    {

	      if (!strcmp ("FONTBOUNDINGBOX", getcol (row, 1)))
		{
		  width = atoi (getcol (row, 2));
		  height = atoi (getcol (row, 3));
		  if (!((width == height || width == height / 2) && (height == 14 || height == 16)))
		    {
		      fprintf (errf, "BDF font \"%s\"is not x14 or x16 size !!!\n", av[fc]);
		    }
		  else
		    initc++;
		}

	      if (!strcmp ("CHARS", getcol (row, 1)))
		{
		  NOFCH = atoi (getcol (row, 2));
		  initc++;
		}
	      if (!strcmp ("FONT", getcol (row, 1)))
		{
/*                for (x = 0; row[x]; x++)
   if (row[x] >= 'a' && row[x] <= 'z')
   row[x] -= 32;
 */
		  for (x = 0, ccnum = -1; x < sizeof (SupChC) / sizeof (SupChC[0]); x++)
		    if (strcsstr (row, SupChC[x].regname))
		      {
			ccnum = x;
			break;
		      }
		  if (ccnum < 0)
		    {
		      fprintf (errf, "%s is not supported!\n", row);
		    }
		  else if (SupChC[ccnum].width)
		    fprintf (errf, "%s already exist!\n", row);
		  else
		    initc++;
		}

	    }

	  if (!strcmp ("ENCODING", getcol (row, 1)))
	    {
	      if (!XBMPOS && initc != 3)
		{
		  fprintf (errf, "\nSomething wrong! Please check your BDF \"%s\"file!\nIgnored !!!\n\n", av[fc]);
		  goto newfont;
		}
	      if (!XBMPOS)
		fprintf (dstf, "/* Font %s */\n\
static char *%s_src[]={\n", SupChC[ccnum].regname, SupChC[ccnum].name);


	      NOFB = 0;
	      CHAR = (unsigned int) atoi (getcol (row, 2));
	      if (!XBMPOS)
		first = CHAR;
	      last = CHAR;
	      fprintf (dstf, "/* %x CHAR */\n\"\\\n", CHAR);
	      KANIND[(unsigned int) (CHAR - first)] = XBMPOS++;
	    }

	  if (!strcmp ("ENDCHAR", getcol (row, 1)))
	    {
	      BMB = 0;
	      fprintf (dstf, "\",\n");
	      printf ("Completed...  %u%%\r", (XBMPOS * 100) / NOFCH);
	    }


	  if (BMB)
	    {
	      if (NOFB)
		{
		  fprintf (dstf, "\\\n");
		}
	      cnvtb = mirror (getcol (row, 1));
	      if (width <= 8)
		fprintf (dstf, "\\%03o", cnvtb);
	      else
		fprintf (dstf, "\\%03o\\%03o", cnvtb / 256, cnvtb % 256);
	      NOFB = 1;
	    }

	  if (!strcmp ("BITMAP", getcol (row, 1)))
	    BMB = 1;

	}

      fprintf (dstf, "0};\n");
      fprintf (dstf, "static int %s_ind[]={\n", SupChC[ccnum].name);
      for (x = 0; x < last - first + 1; x++)
	{
	  fprintf (dstf, "%d,", KANIND[x]);
	  if (x % 10 == 9)
	    fprintf (dstf, "\n");
	}
      fprintf (dstf, "0};\n");
      SupChC[ccnum].width = width;
      SupChC[ccnum].height = height;
      SupChC[ccnum].first = first;
      SupChC[ccnum].last = last;
      SupChC[ccnum].is2bytes = (last / 256) ? 1 : 0;
      SupChC[ccnum].has8bit = ((last / 256) / 128 || (last % 256) / 128) ? 1 : 0;
      fclose (srcf);
      if (NOFCH != XBMPOS)
	{
	  fprintf (errf, "\nSomething wrong! Please check your BDF \"%s\"file!\n", av[fc]);
	  exit (1);
	}
      fprintf (infof, "%25s%15s%18s%7u%7u%7u\n", strrchr (av[fc], '/') + 1, SupChC[ccnum].name, SupChC[ccnum].regname, width, height, NOFCH);
      printf ("\nNumber of defined chars: %d\nNumber of processed chars: %d\n", NOFCH, XBMPOS);
    newfont:
      ;
    }
  fclose (infof);
  fprintf (dstf, "int CII_CC_SIZE = %d;\n", tabsize);
  fprintf (dstf, "struct CharCodesXBM CII_CC[%d] ={\n", tabsize);

  for (x = 0; x < sizeof (SupChC) / sizeof (SupChC[0]); x++)
    {
      if (!SupChC[x].width)
	continue;
      fprintf (dstf, "{\"%s\",\"%s\",\n", SupChC[x].name, SupChC[x].regname);
      fprintf (dstf, "%s_src,%s_ind,\n", SupChC[x].name, SupChC[x].name);
      fprintf (dstf, "%d,%d,\n", SupChC[x].width, SupChC[x].height);
      fprintf (dstf, "%u,%u,\n", SupChC[x].first, SupChC[x].last);
      fprintf (dstf, "%d,%d},\n", SupChC[x].is2bytes, SupChC[x].has8bit);
    }
  fprintf (dstf, "0};\n");
  fclose (dstf);
  printf ("\nDone with creating \"_xbmraw.c\".\n");
  fclose (errf);
  exit (0);
}
